package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysMenuRoleMapper
import com.gxzc.zen.api.sys.model.SysMenuRole
import com.gxzc.zen.api.sys.service.ISysMenuRoleService
import com.gxzc.zen.umps.constant.ZenHttpSession
import com.gxzc.zen.umps.util.ShiroRedisUtil
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

/**
 * <p>
 * 用户菜单表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysMenuRoleServiceImpl : ServiceImpl<SysMenuRoleMapper, SysMenuRole>(), ISysMenuRoleService {
    override fun removeByMenuId(menuId: Long) {
        val condition = SysMenuRole().apply {
            this.menuId = menuId
        }
        baseMapper.physicalDelete(EntityWrapper(condition))
    }

    override fun getMenuIdListByRoleId(roleId: Long): List<Long> {
        val condition = SysMenuRole().apply {
            this.roleId = roleId
        }
        return baseMapper.selectWOLogic(EntityWrapper(condition)).map {
            it.menuId!!
        }
    }

    @Transactional
    override fun updateMenuRole(roleId: Long, menuIdList: List<Long>) {
        // 先获取roleid的所有menuid
        val oldMenuIdList = getMenuIdListByRoleId(roleId)
        val deleteIds = oldMenuIdList.subtract(menuIdList)
        val insertIds = menuIdList.subtract(oldMenuIdList)

        var eff = false
        if (deleteIds.isNotEmpty()) {
            baseMapper.physicalDelete(EntityWrapper<SysMenuRole>().eq("role_id", roleId).`in`("menu_id", deleteIds))
            eff = true
        }

        if (insertIds.isNotEmpty()) {
            val entityList = mutableListOf<SysMenuRole>()
            insertIds.forEach {
                entityList.add(SysMenuRole().apply {
                    this.roleId = roleId
                    this.menuId = it
                })
            }
            this.insertBatch(entityList)
            eff = true
        }

        if (eff) {
            // 更新一下缓存SESSION_KEY_USER_MENU
            ShiroRedisUtil.removeAllSessionsAttributeKey(ZenHttpSession.SESSION_KEY_USER_MENU)
        }
    }

}
