package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.plugins.Page
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserMapper
import com.gxzc.zen.api.sys.model.SysUser
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.api.sys.service.ISysUserService
import com.gxzc.zen.common.contants.PLATFORM
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.PlatformUtil
import org.apache.commons.lang3.RandomStringUtils
import org.apache.shiro.crypto.hash.SimpleHash
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

/**
 * <p>
 * 用户管理 服务实现类
 * </p>
 *
 * @author NorthLan
 * @since 2018-02-06
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysUserServiceImpl : ServiceImpl<SysUserMapper, SysUser>(), ISysUserService {
    companion object {
        private val logger = LoggerFactory.getLogger(SysUserServiceImpl::class.java)
    }

    @Autowired
    private lateinit var userRoleService: ISysUserRoleService

    override fun getUserByAccount(account: String): SysUser? {
        val condition = SysUser().apply {
            this.account = account
        }
        return baseMapper.selectOne(condition)
    }

    override fun getUserListPage(keyword: String?, searchOption: Int?, page: Page<SysUser>, enable: Boolean?): Page<SysUser> {
        val condition = EntityWrapper<SysUser>().apply {
            if (!keyword.isNullOrEmpty() && searchOption != null) {
                when (searchOption) {
                    1 -> {
                        // 角色名称
                        like("account", "%$keyword%")
                    }
                    2 -> {
                        // 角色代码
                        like("username", "%$keyword%")
                    }
                    3 -> {
                        // 角色代码
                        like("staffNo", "%$keyword%")
                    }
                }
            }
            if (enable != null) {
                if (enable) {
                    eq("enable", 1)
                } else {
                    eq("enable", 0)
                }
            }
        }
        page.records = baseMapper.selectWOLogicPage(page, condition)
        return page
    }

    override fun getUserList(keyword: String?, searchOption: Int?, enable: Boolean?): MutableList<SysUser> {
        val condition = EntityWrapper<SysUser>().apply {
            if (!keyword.isNullOrEmpty() && searchOption != null) {
                when (searchOption) {
                    1 -> {
                        // 角色名称
                        like("account", "%$keyword%")
                    }
                    2 -> {
                        // 角色代码
                        like("username", "%$keyword%")
                    }
                    3 -> {
                        // 角色代码
                        like("staffNo", "%$keyword%")
                    }
                }
            }
            if (enable != null) {
                if (enable) {
                    eq("enable", 1)
                } else {
                    eq("enable", 0)
                }
            }
        }
        return baseMapper.selectWOLogic(condition)
    }

    override fun register(entity: SysUser): SysUser {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            //
            val condition = SysUser().apply { this.account = entity.account }
            if (baseMapper.selectCount(EntityWrapper(condition)) > 0) {
                throw ZenException(ZenExceptionEnum.REG_ACCOUNT_EXISTS)
            }
            // 处理一下密码
            if (entity.password.isNullOrEmpty()) {
                throw ZenException(ZenExceptionEnum.REG_PASSWORD_ERROR)
            } else if (entity.password!!.length < 6) {
                throw ZenException(ZenExceptionEnum.REG_PASSWORD_ERROR)
            }
            //
            val salt = RandomStringUtils.randomAlphanumeric(9)
            val hashedPassword = SimpleHash("md5", entity.password, entity.account + salt, 2).toHex()
            //
            entity.salt = salt
            entity.password = hashedPassword
            //
            if (baseMapper.insert(entity) <= 0) {
                throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
            }
        }
        return entity
    }

    override fun modify(entity: SysUser): SysUser {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            // 处理一下密码
            if (entity.password != null && entity.password!!.length < 6) {
                throw ZenException(ZenExceptionEnum.REG_PASSWORD_ERROR)
            }
            if (entity.password != null) {
                //
                val salt = RandomStringUtils.randomAlphanumeric(9)
                val hashedPassword = SimpleHash("MD5", entity.password, entity.account + salt, 2).toHex()
                //
                entity.salt = salt
                entity.password = hashedPassword
            }
            // 修改
            baseMapper.updateWOLogic(entity, EntityWrapper<SysUser>().eq("id", entity.id))
        }
        return entity
    }

    @Transactional
    override fun delete(id: Long) {
        if (baseMapper.physicalDeleteById(id) > 0) {
            // 删除用户相关所有东西
            userRoleService.deleteByUserId(id)
        }
    }
}
